package de.ids_mannheim.korap.openthesaurus;

import java.io.*;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.net.HttpURLConnection;
import java.nio.file.Path;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

public class OpenThesaurusData {

    /**
     * Gets the filesize from a URL of a file to be downloaded.
     * This is necessary to know in cases, where a download is resumed
     * (e.g. after the connection is lost), because we don't want to start
     * downloading from the beginning.
     * @param  url URL of a file to download
     * @return filesize in bytes
     */

    private static long getFileSize(URL url) {
        URLConnection conn = null;
        try {
            conn = url.openConnection();
            if (conn instanceof HttpURLConnection) {
                ((HttpURLConnection) conn).setRequestMethod("HEAD");
            }
            conn.getInputStream();
            return conn.getContentLengthLong();
        } catch (IOException e) {
            throw new RuntimeException(e);
        } finally {
            if(conn instanceof HttpURLConnection) {
                ((HttpURLConnection) conn).disconnect();
            }
        }
    }

    /**
     * A convenience function that combines downloading and extracting a .zip file
     * (e.g. OpenThesaurus data is provided as a .zip-file) at the same time
     * @param URLPath link to the data to download
     * @param thesaurusPath filepath to save the extracted data to
     * @throws IOException
     */
    public static void getFile(String URLPath, String thesaurusPath) throws IOException {
        URL url;
        try {
            url = new URL(URLPath);
        }
        // if a simple file path is passed:
        catch (MalformedURLException e) {
            url = Path.of(URLPath).toUri().toURL();
        }

        File thesaurusFile = new File(thesaurusPath);

        // This creates directories in the filepath if they don't yet exist:
        File newDirs = thesaurusFile.getParentFile();
        if (!(newDirs == null)) thesaurusFile.getParentFile().mkdirs();

        URLConnection urlconnection = url.openConnection();
        urlconnection.setConnectTimeout(10000);
        urlconnection.setReadTimeout(10000);

        FileOutputStream FOS = new FileOutputStream(thesaurusFile, false);

        BufferedOutputStream outputStream = new BufferedOutputStream(FOS);
        urlconnection.connect();
        ZipInputStream zipIn = new ZipInputStream(urlconnection.getInputStream());
        ZipEntry entry = zipIn.getNextEntry();
        while (entry != null) {
            if (!entry.isDirectory()) {
                // if the entry is a file, extracts it
                extractFile(zipIn, outputStream);
            }
            zipIn.closeEntry();
            entry = zipIn.getNextEntry();
        }
        zipIn.close();
        outputStream.flush();
        outputStream.close();
    }


    /**
     * Extracts the data from the ZipInputStream in 8192 byte chunks to a given
     * OutputStream
     * @param zipIn ZipInputStream with the data to extract
     * @param bos OutputStream to extract the data to
     * @throws IOException
     */
    private static void extractFile(ZipInputStream zipIn, OutputStream bos) throws IOException {
        byte[] bytesIn = new byte[8192];
        int read;
        while ((read = zipIn.read(bytesIn)) != -1) {
            bos.write(bytesIn, 0, read);
        }
    }
    // /* // Test functionality:
    public static void main(String[] args) throws IOException {
        String url = "https://www.openthesaurus.de/export/OpenThesaurus-Textversion.zip";
        String thesaurusFilepath = "data/openthesaurus.txt";

        getFile(url, thesaurusFilepath);
    }
    // */
}
